{ nixpkgsOnBuildForBuild
, coreboot-toolchain ? throw "missing"
, payload            ? throw "you must provide a coreboot payload (Linux kernel image or FIT)"
, fmap               ? throw "you must provide an FMAP (flash chip partition table)"
, config             ? throw "you must provide a coreboot .config file"
, initramfs_image    # path to the initramfs `cpio` archive
, iasl               ? null # a specific iasl to use, if needed
}:
let
  version = "4.9";
  inherit (nixpkgsOnBuildForBuild) stdenv lib git python2 ncurses fetchgit;
in
stdenv.mkDerivation {
  pname = "coreboot";
  inherit version;

  src = fetchgit {
    url = "https://review.coreboot.org/coreboot";
    branchName = "${version}";
    rev = "7f520c8fe6fc991df2c4e91f42843d4290744ebb";
    hash = "sha256-lX6QnUS4a/F4Y68qK9i45O4OP+UEjHlCK+YaKJOQLUo=";
    fetchSubmodules = false;
  };

  prePatch = ''
    patchShebangs .
  '';

  patches =
    lib.mapAttrsToList
      (k: v: ./patches/. + "/${k}")
      (builtins.readDir ./patches);

  nativeBuildInputs = [
    git
    python2
    ncurses
  ] ++ coreboot-toolchain;

  enableParallelBuilding = false;   # does not work

  # FIXME: use the nixpkgs `kernel/manual-config.nix` machinery here
  configurePhase = ''
    runHook preConfigure
    cp ${config} .config
    chmod +w .config
    sed -i 's/^CONFIG_FMDFILE=.*//' .config
    echo 'CONFIG_FMDFILE="${fmap}"' >> .config
    sed -i 's/^CONFIG_PAYLOAD_FILE=.*//' .config
    echo 'CONFIG_PAYLOAD_FILE="${payload}"' >> .config
    sed -i 's/^CONFIG_LINUX_INITRD=.*//' .config
    echo 'CONFIG_LINUX_INITRD="${initramfs_image}"' >> .config
  '' + lib.optionalString (iasl != null) ''
    echo CONFIG_ANY_TOOLCHAIN=y >> .config
  '' + ''
    runHook postConfigure
  '';

  preBuild = ''
    mkdir -p build/cbfs/prefix/
  '';

  makeFlags = [
    "build/coreboot.rom"
  ] ++ lib.optionals (iasl != null) [
    "IASL=${iasl}/bin/iasl"
  #] ++ [ "V=1"
  ];

  # see https://review.coreboot.org/c/coreboot/+/12825/ for why this is needed
  postBuild = ''
    build/util/cbfstool/cbfstool build/coreboot.rom add-master-header -r NORMAL
  '';

  dontPatchELF = true;

  installPhase = ''
    runHook preInstall

    mkdir -p $out
    cp build/coreboot.rom $out/

    # note that coreboot's `Makefile` rewrites the `.config`, so we
    # keep a copy of the final version
    grep -v '^#' .config | sort > $out/config

    runHook postInstall
  '';
}
