let
  lib = import <nixpkgs/lib>;
  nixpkgsArgs = { config.allowNonSource = false; };
  nixpkgsOnBuildForBuild  = import <nixpkgs> nixpkgsArgs;

  ownerboot = { hostPlatform?null, overlay }:
    (lib.makeScope lib.callPackageWith (self: {
      inherit lib nixpkgsOnBuildForBuild;

      nixpkgsOnBuildForHost   = import <nixpkgs>
        (nixpkgsArgs // (lib.optionalAttrs (hostPlatform!=null) {
          crossSystem = hostPlatform; }));

      coreboot-toolchain = self.callPackage ./coreboot-toolchain { };

      iasl_20180531 = self.nixpkgsOnBuildForBuild.callPackage ./coreboot/iasl_20180531 { };
      coreboot      = self.callPackage ./coreboot {
        initramfs_image = "${self.initramfs}/initramfs.cpio";
      };
      kernel        = self.callPackage ./kernel { };
      initramfs     = self.callPackage ./initramfs {
        userspace = self.callPackage ./userspace {
          kernelname = "${self.kernel.version}-${self.platform_name}";
        };
      };
    }))
  .overrideScope' overlay;
in {
  # one entry for each supported platform
  kevin = ownerboot (import ./platform/kevin);
  am1i  = ownerboot (import ./platform/am1i);
  kgpe  = ownerboot (import ./platform/kgpe);

  em100 = nixpkgsOnBuildForBuild.callPackage ./util/em100 { };
}


