{
  hostPlatform = (import <nixpkgs/lib>).systems.examples.gnu64;

  overlay = (final: prev: {

    platform_name = "kgpe";

    kernel = 
      final.lib.makeOverridable (prev.kernel.override {
        config = ./linux.config;
        buildTargets = [ "bzImage" ];
      }).overrideAttrs (a: {
        postInstall = (a.postInstall or "") + ''
          cp arch/x86_64/boot/bzImage $out
        '';
      });

    coreboot = final.lib.makeOverridable (prev.coreboot.override {
      iasl = final.iasl_20180531;
      payload = "${final.kernel}/bzImage";
      fmap = ./custom.fmap;
      config = ./coreboot.config;
      coreboot-toolchain = with final.coreboot-toolchain; [ x64 i386 ];
    }).overrideAttrs (a: {
      # KGPE has an FMAP region "BIOS" which overlaps the whole flash;
      # this is needed for some kind of protection mechanism in the
      # SMBIOS.  Unfortunately it causes flashrom to choke.  So we have
      # to feed flashrom a `flashrom.layout` file instead of letting it
      # use the FMAP.
      postInstall = (a.postInstall or "") + ''
        cp src/mainboard/asus/kgpe-d16/cmos.layout $out/
        cp src/mainboard/asus/kgpe-d16/cmos.default $out/
         cat > $out/flashrom.layout <<\EOF
        00000400:007fffff NORMAL
        00800400:00ffffff FALLBACK
        EOF
    '';
    });
  });
}
